CREATE TABLE "events" (
        "id" INTEGER PRIMARY KEY  NOT NULL  UNIQUE,
        "name" TEXT NOT NULL,
        "description" TEXT,
        "location_id" INTEGER,
        "owner_id" INTEGER,
        "imagepath" TEXT,
        "begin_date" DATETIME NOT NULL,
        "repetition_pattern" INTEGER,
        "end_date" DATETIME,
        "created_date" DATETIME,
        "lastchanged_date" DATETIME
);

CREATE TABLE "locations" (
        "id" INTEGER  PRIMARY KEY  NOT NULL  UNIQUE,
        "name" TEXT NOT NULL,
        "address" TEXT
);

CREATE TABLE "users" (
        "id" INTEGER PRIMARY KEY  AUTOINCREMENT  NOT NULL  UNIQUE,
        "username" TEXT NOT NULL  UNIQUE,
        "passwordhash" TEXT NOT NULL,
        "screenname" TEXT NOT NULL,
        "email" TEXT NOT NULL  UNIQUE
);

CREATE TRIGGER fk_locations
BEFORE DELETE on locations
FOR EACH ROW BEGIN
        SELECT CASE
                 WHEN ((SELECT COUNT(*) FROM events WHERE location_id = OLD.id) > 0)
                 THEN RAISE(ABORT, 'Cannot remove location that still has events.')
        END;
END;

CREATE TRIGGER fk_users 
BEFORE DELETE on users
FOR EACH ROW BEGIN
        SELECT CASE
                 WHEN ((SELECT COUNT(*) FROM events WHERE owner_id = OLD.id) > 0)
                 THEN RAISE(ABORT, 'Cannot remove user that still has events.')
        END;
END;
