import sqlite3


class Location(object):
    id = None
    name = None
    address = None

    def __init__(self, name, address=None):
        self.name = name
        self.address = address

    @staticmethod
    def from_dict(d):
        l = Location('', '')
        for key in d.keys():
            l.__dict__[key] = d[key]
        return l

    def db_save(self, conn):
        if self.id is None:
            insert_stmt = "INSERT INTO locations (name, address) VALUES (?,?)"
            cur = conn.cursor()
            cur.execute(insert_stmt, (self.name, self.address))
            conn.commit()
            self.id = cur.lastrowid
        else:
            update_stmt = "UPDATE locations SET name = ?, address = ? \
                    WHERE id = ?"
            cur = conn.cursor()
            cur.execute(update_stmt, (self.name, self.address, self.id))
            conn.commit()

    def db_delete(self, conn):
        delete_stmt = "DELETE FROM locations WHERE id = ?"
        cur = conn.cursor()
        try:
            cur.execute(delete_stmt, (self.id,))
            conn.commit()
        except sqlite3.IntegrityError:
            cur.close
            conn.rollback()
            raise Exception('cannot remove location with existing events associated')

    @staticmethod
    def row_factory(cursor, row):
        l = Location(None, None)
        for idx, col in enumerate(cursor.description):
            l.__dict__[col[0]] = row[idx]
        return l

    @classmethod
    def db_load_single(c, conn, location_id):
        select_stmt = "SELECT * FROM locations WHERE id = ?"
        conn.row_factory = c.row_factory
        cur = conn.cursor()
        cur.execute(select_stmt, (location_id,))
        return cur.fetchone()

    @classmethod
    def db_load_all(c, conn):
        select_stmt = "SELECT * FROM locations"
        conn.row_factory = c.row_factory
        cur = conn.cursor()
        cur.execute(select_stmt)
        return cur.fetchall()
