from datetime import timedelta, datetime
from copy import copy


class Repetition(object):
    """
    Baseclass for repetitionsm subclass to add new one. Don't forget to fill
    in the unfold_event method.
    """

    @staticmethod
    def copy_event(event, offset):
        new_event = copy(event)
        new_event.begin_date += offset
        if new_event.end_date:
            new_event.end_date += offset

        return new_event

class NoRepetition(Repetition):
    descriptor = "Keine Wiederholung"

    def __init__(self):
        Repetition.__init__(self)

    def unfold_events(self, event, until=None):
        return []


class WeeklyRepetition(Repetition):
    descriptor = "Woechentliche Wiederholung"
    one_week = timedelta(weeks=1)

    def unfold_events(self, event, until=None):
        if until is None:
            until = datetime.now() + timedelta(weeks=12)

        current_offset = self.one_week
        add_events = []
        while event.begin_date + current_offset < until:
            add_events.append(Repetition.copy_event(event, current_offset))
            current_offset += self.one_week

        return add_events


class BiweeklyRepetition(Repetition):
    descriptor = "Zweiwoechentliche Wiederholung"
    two_weeks = timedelta(weeks=2)

    def unfold_events(self, event, until=None):
        if until is None:
            until = datetime.now() + timedelta(weeks=12)

        current_offset = self.two_weeks
        add_events = []
        while event.begin_date + current_offset < until:
            add_events.append(Repetition.copy_event(event, current_offset))
            current_offset += self.two_weeks

        return add_events


repetition_class_by_id = [
    NoRepetition,
    WeeklyRepetition,
    BiweeklyRepetition
]
