from werkzeug.security import generate_password_hash, \
    check_password_hash


class User(object):
    id = None
    username = None
    passwordhash = None
    screenname = None
    email = None

    def __init__(self, username, password, screenname, email):
        self.username = username
        self.passwordhash = generate_password_hash(password)
        self.screenname = screenname
        self.email = email

    def set_password(self, password):
            self.passwordhash = generate_password_hash(password)

    def check_password(self, password):
        return check_password_hash(self.passwordhash, password)

    @staticmethod
    def from_dict(d):
        u = User('', '', '', '')
        for key in d.keys():
            u.__dict__[key] = d[key]
        return u

    def db_save(self, conn):
        if self.id is None:
            insert_stmt = "INSERT INTO users (username, passwordhash, \
                    screenname, email) VALUES (?,?,?,?)"
            cur = conn.cursor()
            cur.execute(insert_stmt, (self.username, self.passwordhash,
                                      self.screenname, self.email))
            self.id = cur.lastrowid
            conn.commit()
        else:
            update_stmt = "UPDATE users SET username = ?, passwordhash = ?, \
                    screenname = ?, email = ? WHERE id = ?"
            cur = conn.cursor()
            cur.execute(update_stmt, (self.username, self.passwordhash,
                                      self.screenname, self.email, self.id))
            conn.commit()

    def db_delete(self, conn):
        delete_stmt = "DELETE FROM users WHERE id = ?"
        cur = conn.cursor()
        cur.execute(delete_stmt, (self.id,))
        conn.commit()

    @staticmethod
    def row_factory(cursor, row):
        u = User(None, '', None, None)
        for idx, col in enumerate(cursor.description):
            u.__dict__[col[0]] = row[idx]
        return u

    @classmethod
    def db_load_single(c, conn, user_id=None, username=None):
        print "asdf"
        conn.row_factory = c.row_factory
        if user_id is None and username is None:
            raise Exception('No identifying Information about the user has \
                            been passed')
        else:
            cur = conn.cursor()
            if not user_id is None:
                select_stmt = "SELECT * FROM users WHERE id = ?"
                cur.execute(select_stmt, (user_id,))
            else:
                select_stmt = "SELECT * FROM users WHERE username = ?"
                cur.execute(select_stmt, (username,))

        return cur.fetchone()

    @classmethod
    def db_load_all(c, conn):
        conn.row_factory = c.row_factory
        cur = conn.cursor()
        select_stmt = "SELECT * from users"
        cur.execute(select_stmt)
        return cur.fetchall()
