import sqlite3
from datetime import datetime

from model.location import Location
from model.user import User
from model.event import Event
conn = None


def setup_module():
    global conn
    conn = sqlite3.connect(':memory:')
    cur = conn.cursor()
    cur.executescript(open('events.sql').read())
    conn.commit()


def teardown_module():
    global conn
    conn.close()


def test_locations_database():
    """
    Database lifecycle of a location Object
    """
    global conn
    l = Location('asdf', 'asdf')
    l.db_save(conn)
    cur = conn.cursor()
    cur.execute("SELECT COUNT(id) FROM locations WHERE id = ?", (l.id,))
    assert cur.fetchone()[0] == 1

    l.name = 'lel'
    l.db_save(conn)
    cur = conn.cursor()
    cur.execute("SELECT name FROM locations WHERE id = ?", (l.id,))
    assert cur.fetchone()[0] == 'lel'

    l.db_delete(conn)
    cur.execute("SELECT COUNT(id) FROM locations WHERE id = ?", (l.id,))
    assert cur.fetchone()[0] == 0


def test_users_database():
    """
    Database lifecycle of a user Object
    """
    global conn
    u = User(username='asdf', password='lel', screenname='asdf',
             email='lel@example.com')
    u.db_save(conn)
    cur = conn.cursor()
    cur.execute("SELECT COUNT(id) FROM users WHERE id = ?", (u.id,))
    assert cur.fetchone()[0] == 1

    u.username = 'lel'
    u.db_save(conn)
    cur = conn.cursor()
    cur.execute("SELECT username FROM users WHERE id = ?", (u.id,))
    assert cur.fetchone()[0] == 'lel'

    u.db_delete(conn)
    cur = conn.cursor()
    cur.execute("SELECT COUNT(id) FROM users WHERE id = ?", (u.id,))
    assert cur.fetchone()[0] == 0


def test_event_database():
    """
    Database lifecycle of a event object
    """

    global conn
    u = User(username='asdf', password='lel', screenname='asdf',
             email='lel@example.com')
    u.db_save(conn)
    l = Location('asdf', 'asdf')
    l.db_save(conn)
    e = Event(name='lel', begin_date=datetime.now(), location=l, owner=u)
    e.db_save(conn)

    cur = conn.cursor()
    cur.execute("SELECT id, location_id, owner_id FROM events WHERE id = ?", (e.id,))
    res = cur.fetchone()
    assert res[0] == e.id
    assert res[1] == l.id
    assert res[2] == u.id

    f = Event.db_load_single(conn, event_id=e.id)
    assert f.created_date is not None
    assert f.lastchanged_date is not None

    f.name = 'lel'
    f.repetition_pattern = 1
    f.db_save(conn)
    assert f.lastchanged_date > e.lastchanged_date

    g = Event.db_load_single(conn, event_id=f.id)
    assert g.repetition_pattern == 1
