from datetime import datetime, timedelta
import sys
sys.path.append("../")

from werkzeug.security import check_password_hash

from model.user import User
from model.event import Event
from model.location import Location


def test_user_creation():
    u = User('test', 'test', 'Testuser', 'test@example.com')
    assert u.username == 'test'
    assert check_password_hash(u.passwordhash, 'test')
    assert u.screenname == 'Testuser'
    assert u.email == 'test@example.com'


def test_user_change_password():
    u = User('test', 'test', 'Testuser', 'test@example.com')
    u.set_password('new')
    assert check_password_hash(u.passwordhash, 'new')


def test_user_checkpassword():
    u = User('test', 'test', 'Testuser', 'test@example.com')
    u.set_password('new')
    assert u.check_password('new')


def test_event_creation():
    e = Event('Testevent', datetime.now())
    assert e.name == 'Testevent'

def test_location_creation():
    l = Location('Testlocation')
    assert l.name == 'Testlocation'

def test_event_repetition():
    l = Location('asdf')
    u = User('test', 'test', 'Testuser', 'test@example.com')
    e = Event('Testevent', datetime.now())
    e.repetition_pattern = 1

    in_5_hours = datetime.now() + timedelta(hours=5)
    lel = Event.resolve_repetions([e], before=in_5_hours)
    assert len(lel) > 1
