import os
from werkzeug.utils import redirect
from werkzeug.wrappers import Response
from PIL import Image
from StringIO import StringIO
from hashlib import md5

from model.user import User


def render_template(template_name, environment, mimetype='text/html',
                    **kwargs):
    """
    renders a template in the current environment and returns it
    wrapped in a werkzeug Response object.
    """
    jinja_env = environment['EventManagement.jinja_env']
    template = jinja_env.get_template(template_name)
    return Response(template.render(**kwargs), mimetype=mimetype)


def gravatar_url(email, size):
    """
    generates the gravatar URL
    """
    import urllib
    import hashlib

    # Set your variables here
    default = "http://www.openlab-augsburg.de/_media/openwiki:bootstrap:logo-\
            symbol-only.png"

    # construct the url
    gravatar_url = "http://www.gravatar.com/avatar.php?"
    gravatar_url += urllib.urlencode(
        {'gravatar_id': hashlib.md5(email.lower()).hexdigest(),
         'default': default,
         'size': str(size)})

    return gravatar_url


def authenticated(function):
    """
    checks if a user is authenticated, redirects to login view otherwise
    """
    def inner(*args, **kwargs):
        environment = args[1]
        session = environment['beaker.session']
        if "username" not in session.keys():
            return redirect('/login')
        conn = args[2]
        u = User.db_load_single(conn, username=session['username'])
        if u is None:
            session.delete()
            return redirect('/login')
        return function(*args, **kwargs)

    return inner


def force_quadratic(image):
    """
    Forces quadratic geometry on an image
    """
    (width, height) = image.size
    if width == height:
        return image
    size = min(width, height)
    delta = (max(width, height) - size) / 2
    if width < height:
        box = (0, delta, size, size + delta)
    else:
        box = (delta, 0, size + delta, size)
    image = image.crop(box)
    return image


def check_mimetype(mimetype, supported_maintypes, supported_subtypes):
    """
    Checks whether the given mimetype is supported
    """
    mimesplit = mimetype.split('/')
    if mimesplit[0] not in supported_maintypes or mimesplit[1] not in\
            supported_subtypes:
        raise Exception("Unsupported mimetype: " + mimetype + ",\
                        expected " + supported_maintypes + "/" +
                        supported_subtypes)


def save_image(file_obj):
    # determine file type and check if it is allowed
    mime_type = file_obj.content_type
    check_mimetype(mime_type, ["image"], ["jpeg", "gif", "png"])
    file_ending = mime_type.split('/')[1]

    # fit geometry
    image_buf = file_obj.read()
    image = Image.open(StringIO(image_buf))

    image = force_quadratic(image)

    filename = md5(image_buf).hexdigest() + '.' + file_ending
    fullpath = os.path.join('assets', 'event-images', filename)

    image.save(fullpath)

    return '/assets/event-images/' + filename
