from werkzeug.utils import redirect
from wtforms import Form, TextField, FileField, TextAreaField, DateTimeField,\
    SelectField, validators

from util import render_template, authenticated
from model.location import Location


class LocationForm(Form):
    name = TextField("Name", [validators.required()])
    address = TextField("Adresse", [validators.required()])


@authenticated
def web_add_location(request, environment, conn):
    if request.method != "POST":
        form = LocationForm()
        return render_template("location/web_add_location.html", environment,
                               form=form)
    else:
        form = LocationForm(request.form)
        if form.validate():
            l = Location(form.name.data, form.address.data)
            l.db_save(conn)
            return redirect('/admin')
        else:
            return render_template("location/web_add_location.html",
                                   environment, form=form)


@authenticated
def web_delete_location(request, environ, conn, location_id):
    location_obj = Location.db_load_single(conn,location_id=location_id)
    if not location_obj is None:
        # TODO: Don't delete if it has Events attached.
        location_obj.db_delete(conn)
    return redirect('/admin')


@authenticated
def web_edit_location(request, environment, conn, location_id):
    location_obj = Location.db_load_single(conn, location_id=location_id)

    if request.method != "POST":
        form = LocationForm()
        form.name.data = location_obj.name
        form.address.data = location_obj.address
        return render_template("location/web_edit_location.html", environment,
                               form=form)
    else:
        form = LocationForm(request.form)
        if form.validate():
            location_obj.name = form.name.data
            location_obj.address = form.address.data
            location_obj.db_save(conn)
            return redirect('/admin')
        else:
            return render_template("location/web_edit_location.html",
                                   environment, form=form)
