from werkzeug.utils import redirect
from wtforms import Form, TextField, PasswordField, validators

from util import render_template, gravatar_url, authenticated
from model.user import User


class UserForm(Form):
    username = TextField("Benutzername", [validators.required()])
    password = PasswordField("Passwort")
    screenname = TextField("Anzeigename")
    email = TextField("E-Mail-Adresse", [validators.Email()])


def web_login(request, environment, conn):
    class LoginForm(Form):
        username = TextField('Username', [validators.Required()])
        password = PasswordField('Passwort', [validators.Required()])

    if request.method == "POST":
        form = LoginForm(request.form)

        if form.validate():
            u = User.db_load_single(conn, username=form.username.data)
            if u is None or not u.check_password(form.password.data):
                form.errors['Login'] = ['Benutzer existiert nicht oder\
                                        Passwort ist falsch.']
            else:
                session = environment['beaker.session']
                session['username'] = u.username
                session.save()
                return redirect('/admin')

        return render_template("user/login.html", environment, form=form)

    else:
        form = LoginForm()
        return render_template("user/login.html", environment, form=form)


@authenticated
def web_logout(request, environment, conn):
    session = environment['beaker.session']
    session.delete()
    return redirect('/')


def web_list_users(request, environment, conn):
    users = User.db_load_all(conn)
    return render_template("user/web_list_users.html", environment,
                           users=users)


def web_show_user_details(request, environment, conn, user_id):
    u = User.db_load_single(conn, user_id=user_id)
    u.gravatar_url = gravatar_url(u.email, 150)
    return render_template("user/web_show_user_details.html", environment,
                           user=u)


@authenticated
def web_add_user(request, environment, conn):
    if request.method != 'POST':
        form = UserForm()
        return render_template("user/web_add_user.html", environment,
                               form=form)
    else:
        form = UserForm(request.form)
        if form.validate():
            u = User(username=form.username.data,
                     password=form.password.data,
                     email=form.email.data,
                     screenname=form.screenname.data)

            u.db_save(conn)
            return redirect('/admin')
        else:
            return render_template("user/web_add_user.html", environment,
                                   form=form)


@authenticated
def web_edit_user(request, environment, conn, user_id):
    u = User.db_load_single(conn, user_id=user_id)
    if request.method != 'POST':
        form = UserForm()
        form.username.data = u.username
        form.screenname.data = u.screenname
        form.email.data = u.email

        return render_template("user/web_edit_user.html", environment,
                               form=form)
    else:
        form = UserForm(request.form)
        if form.validate():
            u.username = form.username.data
            if form.username.data != "":
                u.set_password(form.password.data)
            u.screenname = form.screenname.data
            u.email = form.email.data

            u.db_save(conn)
            return redirect('/admin')
        else:
            return render_template("user/web_edit_user.html", environment,
                                   form=form)


@authenticated
def web_delete_user(request, environment, conn, user_id):
    u = User.db_load_single(conn, user_id=user_id)
    u.db_delete(conn)
    return redirect('/admin')
